@@ -33,6 +33,7 @@
 #include <gio/gio.h>
 #include <gtk/gtk.h>
 #include <glib/gi18n.h>
+#include <libhandy-1/handy.h>
 #include <hb.h>
 #include <hb-ot.h>
 #include <hb-ft.h>
@@ -44,7 +45,7 @@
 #include "sushi-font-widget.h"
 
 #define FONT_VIEW_TYPE_APPLICATION (font_view_application_get_type ())
-#define FONT_VIEW_ICON_NAME "org.gnome.font-viewer"
+#define FONT_VIEW_ICON_NAME APPLICATION_ID
 
 G_DECLARE_FINAL_TYPE (FontViewApplication, font_view_application,
                       FONT_VIEW, APPLICATION,
@@ -260,12 +261,15 @@ font_view_item_load_thumbnail_job (GTask *task,
 
         if (error != NULL) {
             g_debug ("Can't query info for file %s: %s", file_uri, error->message);
+            g_task_return_error (task, g_steal_pointer(&error));
             return;
         }
 
         thumb_failed = g_file_info_get_attribute_boolean (info, G_FILE_ATTRIBUTE_THUMBNAILING_FAILED);
-        if (thumb_failed)
+        if (thumb_failed) {
+            g_task_return_pointer (task, NULL, NULL);
             return;
+        }
 
         thumb_path = g_strdup (g_file_info_get_attribute_byte_string (info, G_FILE_ATTRIBUTE_THUMBNAIL_PATH));
     } else {
@@ -296,6 +300,7 @@ font_view_item_load_thumbnail_job (GTask *task,
 
         if (error != NULL) {
             g_debug ("Can't read file %s: %s", thumb_path, error->message);
+            g_task_return_error (task, g_steal_pointer(&error));
             return;
         }
 
@@ -306,6 +311,7 @@ font_view_item_load_thumbnail_job (GTask *task,
 
         if (error != NULL) {
             g_debug ("Can't read thumbnail pixbuf %s: %s", thumb_path, error->message);
+            g_task_return_error (task, g_steal_pointer(&error));
             return;
         }
     } else {
@@ -1033,8 +1039,14 @@ font_widget_loaded_cb (SushiFontWidget *font_widget,
     uri = sushi_font_widget_get_uri (font_widget);
     self->font_file = g_file_new_for_uri (uri);
 
-    gtk_header_bar_set_title (GTK_HEADER_BAR (self->header), face->family_name);
-    gtk_header_bar_set_subtitle (GTK_HEADER_BAR (self->header), face->style_name);
+    if (face->family_name) {
+        hdy_header_bar_set_title (HDY_HEADER_BAR (self->header), face->family_name);
+    } else {
+        g_autofree gchar *basename = g_file_get_basename (self->font_file);
+        hdy_header_bar_set_title (HDY_HEADER_BAR (self->header), basename);
+    }
+
+    hdy_header_bar_set_subtitle (HDY_HEADER_BAR (self->header), face->style_name);
 
     install_button_refresh_appearance (self, NULL);
 }
@@ -1150,7 +1162,7 @@ font_view_application_do_open (FontViewApplication *self,
         gtk_widget_set_valign (self->install_button, GTK_ALIGN_CENTER);
         gtk_style_context_add_class (gtk_widget_get_style_context (self->install_button),
                                      "text-button");
-        gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header), self->install_button);
+        hdy_header_bar_pack_end (HDY_HEADER_BAR (self->header), self->install_button);
 
         g_signal_connect (self->install_button, "clicked",
                           G_CALLBACK (install_button_clicked_cb), self);
@@ -1161,7 +1173,7 @@ font_view_application_do_open (FontViewApplication *self,
         gtk_widget_set_valign (self->info_button, GTK_ALIGN_CENTER);
         gtk_style_context_add_class (gtk_widget_get_style_context (self->info_button),
                                      "text-button");
-        gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header), self->info_button);
+        hdy_header_bar_pack_end (HDY_HEADER_BAR (self->header), self->info_button);
 
         g_signal_connect (self->info_button, "toggled",
                           G_CALLBACK (info_button_clicked_cb), self);
@@ -1178,7 +1190,7 @@ font_view_application_do_open (FontViewApplication *self,
         gtk_widget_set_valign (self->back_button, GTK_ALIGN_CENTER);
         gtk_style_context_add_class (gtk_widget_get_style_context (self->back_button),
                                      "image-button");
-        gtk_header_bar_pack_start (GTK_HEADER_BAR (self->header), self->back_button);
+        hdy_header_bar_pack_start (HDY_HEADER_BAR (self->header), self->back_button);
 
         gtk_actionable_set_action_name (GTK_ACTIONABLE (self->back_button), "app.back");
     }
@@ -1252,8 +1264,8 @@ font_view_application_do_overview (FontViewApplication *self)
 
     font_view_ensure_model (self);
 
-    gtk_header_bar_set_title (GTK_HEADER_BAR (self->header), _("All Fonts"));
-    gtk_header_bar_set_subtitle (GTK_HEADER_BAR (self->header), NULL);
+    hdy_header_bar_set_title (HDY_HEADER_BAR (self->header), _("All Fonts"));
+    hdy_header_bar_set_subtitle (HDY_HEADER_BAR (self->header), NULL);
 
     if (self->flow_box == NULL) {
         GtkWidget *flow_box;
@@ -1301,6 +1313,12 @@ font_view_window_key_press_event_cb (GtkWidget *widget,
         return TRUE;
     }
 
+    if (event->keyval == GDK_KEY_f &&
+        (event->state & GDK_CONTROL_MASK) != 0) {
+        gtk_search_bar_set_search_mode (GTK_SEARCH_BAR (self->search_bar), TRUE);
+        return TRUE;
+    }
+
     return gtk_search_bar_handle_event (GTK_SEARCH_BAR (self->search_bar), (GdkEvent *)event);
 }
 
@@ -1366,12 +1384,13 @@ action_about (GSimpleAction *action,
                            "authors", authors,
                            "program-name", _("Fonts"),
                            "comments", _("View fonts on your system"),
+                           "website", "https://gitlab.gnome.org/GNOME/gnome-font-viewer/",
                            "logo-icon-name", FONT_VIEW_ICON_NAME,
                            "translator-credits", _("translator-credits"),
                            "license-type", GTK_LICENSE_GPL_2_0,
                            "wrap-license", TRUE,
                            NULL);
-                           
+
 }
 
 static void
@@ -1406,21 +1425,22 @@ ensure_window (FontViewApplication *self)
     if (self->main_window)
         return;
 
-    self->main_window = window = gtk_application_window_new (GTK_APPLICATION (self));
+    self->main_window = window = hdy_application_window_new ();
+    gtk_window_set_application (GTK_WINDOW (window), GTK_APPLICATION (self));
     gtk_window_set_resizable (GTK_WINDOW (window), TRUE);
     gtk_window_set_default_size (GTK_WINDOW (window), 800, 600);
     gtk_window_set_icon_name (GTK_WINDOW (window), FONT_VIEW_ICON_NAME);
 
-    self->header = gtk_header_bar_new ();
-    gtk_header_bar_set_show_close_button (GTK_HEADER_BAR (self->header), TRUE);
+    self->header = hdy_header_bar_new ();
+    hdy_header_bar_set_show_close_button (HDY_HEADER_BAR (self->header), TRUE);
     gtk_style_context_add_class (gtk_widget_get_style_context (GTK_WIDGET (self->header)),
                                  "titlebar");
-    gtk_window_set_titlebar (GTK_WINDOW (self->main_window), self->header);
 
     g_signal_connect (window, "key-press-event",
                       G_CALLBACK (font_view_window_key_press_event_cb), self);
 
     self->main_grid = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+    gtk_container_add (GTK_CONTAINER (self->main_grid), self->header);
     gtk_container_add (GTK_CONTAINER (self->main_window), self->main_grid);
 
     self->stack = gtk_stack_new ();
@@ -1443,7 +1463,7 @@ ensure_window (FontViewApplication *self)
     gtk_menu_button_set_menu_model (GTK_MENU_BUTTON (self->menu_button), menu);
     gtk_widget_set_no_show_all (self->menu_button, TRUE);
     gtk_widget_show (self->menu_button);
-    gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header), self->menu_button);
+    hdy_header_bar_pack_end (HDY_HEADER_BAR (self->header), self->menu_button);
 
     self->search_bar = gtk_search_bar_new ();
     gtk_container_add (GTK_CONTAINER (box), self->search_bar);
@@ -1456,7 +1476,7 @@ ensure_window (FontViewApplication *self)
     image = gtk_image_new_from_icon_name ("edit-find-symbolic", GTK_ICON_SIZE_BUTTON);
     gtk_widget_show (image);
     gtk_container_add (GTK_CONTAINER (self->search_toggle), image);
-    gtk_header_bar_pack_end (GTK_HEADER_BAR (self->header), self->search_toggle);
+    hdy_header_bar_pack_end (HDY_HEADER_BAR (self->header), self->search_toggle);
     g_object_bind_property (self->search_bar, "search-mode-enabled",
                             self->search_toggle, "active",
                             G_BINDING_BIDIRECTIONAL);
@@ -1488,6 +1508,8 @@ font_view_application_startup (GApplication *application)
 
     G_APPLICATION_CLASS (font_view_application_parent_class)->startup (application);
 
+    hdy_init ();
+
     if (!FcInit ())
         g_critical ("Can't initialize fontconfig library");
 
@@ -1546,7 +1568,7 @@ static GApplication *
 font_view_application_new (void)
 {
     return g_object_new (FONT_VIEW_TYPE_APPLICATION,
-                         "application-id", "org.gnome.font-viewer",
+                         "application-id", APPLICATION_ID,
                          "flags", G_APPLICATION_HANDLES_OPEN,
                          NULL);
 }
